@@ -12,3 +12,7 @@ instance Stream ByteString Word8 where
     peek = head
     atEnd = null
     consume = tail
+
+instance Listish ByteString Word8 where
+    toList = unpack
+    fromList = pack
