@@ -21,3 +21,11 @@ class Token t where
 instance Token Char where
     isLineDelimiter '\n' = True
     isLineDelimiter _ = False
+
+class Listish s t | s -> t where
+    toList :: s -> [t]
+    fromList :: [t] -> s
+
+instance Listish [a] a where
+    toList = id
+    fromList = id
